
--
-- AnimatedHydraulic
--
-- @author  Manuel Leithner
-- @date  12/04/10
--

AnimatedHydraulic = {};

function AnimatedHydraulic.prerequisitesPresent(specializations)
    return true;
end;

function AnimatedHydraulic:load(xmlFile)

	self.setHydraulicDirection = SpecializationUtil.callSpecializationsFunction("setHydraulicDirection");
	
	local i=0;
	self.hydraulics = {};	
	while true do
		local hydraulicName = string.format("vehicle.hydraulics.hydraulic(%d)", i);		
		local hydraulic = {};
		local index = getXMLString(xmlFile, hydraulicName .. "#index")
		if index == nil then
			break;
		end;
		hydraulic.node = Utils.indexToObject(self.components, index);
		hydraulic.punch = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#punch"));
		hydraulic.translationPunch = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#punchFixpoint"));
		hydraulic.fixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#fixpoint"));	
		local xUp, yUp, zUp = Utils.getVectorFromString(Utils.getNoNil(getXMLString(xmlFile, hydraulicName .. "#upVectors"),"0 1 0"));
		hydraulic.upVectors = {xUp, yUp, zUp};
		
		local ax, ay, az;		
		if hydraulic.punch ~= nil then
			ax, ay, az = getWorldTranslation(hydraulic.punch);
		else
			ax, ay, az = getWorldTranslation(hydraulic.node);
		end;
		if hydraulic.translationPunch ~= nil then
			local bx, by, bz = getWorldTranslation(hydraulic.translationPunch);		
			hydraulic.punchDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);
		end;
		hydraulic.needsUpdate = true;
		hydraulic.ongoingUpdate = Utils.getNoNil(getXMLBool(xmlFile, hydraulicName .. "#ongoingUpdate", false));
		
		table.insert(self.hydraulics, hydraulic);
		i = i + 1;
	end;
	self.setHydraulicTime = 30;
end;

function AnimatedHydraulic:delete()
end;

function AnimatedHydraulic:mouseEvent(posX, posY, isDown, isUp, button)
end;

function AnimatedHydraulic:keyEvent(unicode, sym, modifier, isDown)
end;

function AnimatedHydraulic:update(dt)

	if self:getIsActive() then			
		self.setHydraulicTime = 30;	
	end;

	if self.setHydraulicTime > 0 then
		for k,v in pairs(self.hydraulics) do 
			self:setHydraulicDirection(k);
		end;
		self.setHydraulicTime = self.setHydraulicTime - 1;
	end;
end;

function AnimatedHydraulic:draw()	
end;

function AnimatedHydraulic:setHydraulicDirection(index)
	local hydraulic = self.hydraulics[index];

	if hydraulic.needsUpdate and hydraulic.fixPoint ~= nil then
		local ax, ay, az = getWorldTranslation(hydraulic.node);
		local bx, by, bz = getWorldTranslation(hydraulic.fixPoint);
		local x, y, z = worldDirectionToLocal(getParent(hydraulic.node), bx-ax, by-ay, bz-az);
		local xUp, yUp, zUp = unpack(hydraulic.upVectors);
		setDirection(hydraulic.node, x, y, z, xUp, yUp, zUp);
		if hydraulic.punch ~= nil then
			local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
			setTranslation(hydraulic.punch, 0, 0, distance-hydraulic.punchDistance);
		else
			if hydraulic.translationPunch ~= nil then
				local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
				setScale(hydraulic.node, 1, 1, distance / hydraulic.punchDistance);
			end;
		end;
		
		if not hydraulic.ongoingUpdate then
			hydraulic.needsUpdate = false;
		end;
	end;
end;
