--
-- ExtendedWindrower
-- Specialization for ExtendedWindrower
--
-- @author  Manuel Leithner
-- @date  05/07/10
--

ExtendedWindrower = {};

function ExtendedWindrower.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Attachable, specializations);
end;

function ExtendedWindrower:load(xmlFile)

	self.setActivity = SpecializationUtil.callSpecializationsFunction("setActivity");	
	
	self.groundReferenceThreshold = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.groundReferenceNode#threshold"), 0.2);

    self.animation = {};
    self.animation.animCharSet = 0;
    self.animationEnabled = false;
	self.animationSpeed = 0;
    local rootNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.animation#rootNode"));

    if rootNode ~= nil then
        self.animation.animCharSet = getAnimCharacterSet(rootNode);
        if self.animation.animCharSet ~= 0 then
            self.animation.clip = getAnimClipIndex(self.animation.animCharSet, getXMLString(xmlFile, "vehicle.animation#animationClip"));
            if self.animation.clip >= 0 then
                assignAnimTrackClip(self.animation.animCharSet, 0, self.animation.clip);
                self.animation.speedScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.animation#speedScale"), 1);
                setAnimTrackSpeedScale(self.animation.animCharSet, self.animation.clip, self.animation.speedScale);
                setAnimTrackLoopState(self.animation.animCharSet, 0, true);
            end;
        end;
		self.animation.offsetTime = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.animation#offsetTime"), 3) * 1000;
    end;

	local numCuttingAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.cuttingAreas#count"), 0);
    for i=1, numCuttingAreas do
        local areanamei = string.format("vehicle.cuttingAreas.cuttingArea%d", i);
        self.cuttingAreas[i].foldMinLimit = Utils.getNoNil(getXMLFloat(xmlFile, areanamei .. "#foldMinLimit"), 0);
        self.cuttingAreas[i].foldMaxLimit = Utils.getNoNil(getXMLFloat(xmlFile, areanamei .. "#foldMaxLimit"), 1);
		self.cuttingAreas[i].isExpanded = false;
		self.cuttingAreas[i].groundReferenceNode = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#groundReferenceNode"));
    end;
	
    local numWindrowerDropAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.windrowerDropAreas#count"), 0);
    self.windrowerDropAreas = {}
    for i=1, numWindrowerDropAreas do
        self.windrowerDropAreas[i] = {};
        local areanamei = string.format("vehicle.windrowerDropAreas.windrowerDropArea%d", i);
        self.windrowerDropAreas[i].start = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#startIndex"));
        self.windrowerDropAreas[i].width = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#widthIndex"));
        self.windrowerDropAreas[i].height = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#heightIndex"));
    end;
	
    local windrowerSound = getXMLString(xmlFile, "vehicle.windrowerSound#file");
    if windrowerSound ~= nil and windrowerSound ~= "" then
        windrowerSound = Utils.getFilename(windrowerSound, self.baseDirectory);
        self.windrowerSound = createSample("windrowerSound");
        self.windrowerSoundEnabled = false;
        loadSample(self.windrowerSound, windrowerSound, false);
        self.windrowerSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.windrowerSound#pitchOffset"), 1);
        self.windrowerSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.windrowerSound#volume"), 1.0);
    end;
	
    local windrowerStartSound = getXMLString(xmlFile, "vehicle.windrowerStartSound#file");
    if windrowerStartSound ~= nil and windrowerStartSound ~= "" then
        windrowerStartSound = Utils.getFilename(windrowerStartSound, self.baseDirectory);
        self.windrowerStartSound = createSample("windrowerStartSound");
        loadSample(self.windrowerStartSound, windrowerStartSound, false);
        self.windrowerStartSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.windrowerStartSound#pitchOffset"), 1);
        self.windrowerStartSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.windrowerStartSound#volume"), 1.0);
    end;
	
    local windrowerStopSound = getXMLString(xmlFile, "vehicle.windrowerStopSound#file");
    if windrowerStopSound ~= nil and windrowerStopSound ~= "" then
        windrowerStopSound = Utils.getFilename(windrowerStopSound, self.baseDirectory);
        self.windrowerStopSound = createSample("windrowerStopSound");
        loadSample(self.windrowerStopSound, windrowerStopSound, false);
        self.windrowerStopSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehiclewindrowerStopSound#pitchOffset"), 1);
        self.windrowerStopSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.windrowerSound#volume"), 1.0);
    end;

	self.particleSystems = {};
    local i=0;
    while true do
        local baseName = string.format("vehicle.particleSystems.particleSystem(%d)", i);

        local particleSystem = {};
        particleSystem.ps = {};
        local ps = Utils.loadParticleSystem(xmlFile, particleSystem.ps, baseName, self.components[2].node, false, nil, self.baseDirectory)
        if ps == nil then
            break;
        end;
        particleSystem.disableTime = 0;
        table.insert(self.particleSystems, particleSystem);
        i = i+1;
    end;


	local i=0;
	local invisible = 0;
	self.tines = {};
	self.transportTime = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.tines#transportTime"), 3) * 1000;
	while true do
		local tineName = string.format("vehicle.tines.tine(%d)", i);
		
		local nodeString = getXMLString(xmlFile, tineName .. "#index");
		if nodeString ==  nil then
			break;
		end;	
		local tine = {};		
		tine.node = Utils.indexToObject(self.components, nodeString);	
		tine.marker = Utils.getNoNil(getXMLInt(xmlFile, tineName .. "#marker"), 1);
		if getXMLBool(xmlFile, tineName .. "#isVisible") ~= nil then
			invisible = invisible + 1;
		end;
		
		table.insert(self.tines, tine);
		i = i + 1;
	end;
	
	self.invisibleTineCount = invisible;
	self.timePerTine = self.transportTime / (table.getn(self.tines)-self.invisibleTineCount);
	self.timeToNextTine = self.timePerTine;
	self.tinePositions = {};
	i=0;
	while true do
		local positionName =  string.format("vehicle.tines.transportPositions.position(%d)", i);
		local position = getXMLString(xmlFile, positionName .. "#node");
		if position == nil then
			break;
		end;
		position = Utils.indexToObject(self.components, position);
		table.insert(self.tinePositions, position);
		i = i + 1;
	end;
	self.markers = {};
	i=0;
	while true do
		local markerName =  string.format("vehicle.tines.markers.marker(%d)", i);
		local markerNode = getXMLString(xmlFile, markerName .. "#node");
		if markerNode == nil then
			break;
		end;
		marker = {};
		marker.node = Utils.indexToObject(self.components, markerNode);
		local width, _, _ = getTranslation(marker.node);
		marker.width = width;
		marker.scale = Utils.getNoNil(getXMLInt(xmlFile, markerName .. "#scale"), 1);
		table.insert(self.markers, marker);
		i = i + 1;
	end;

	self.doChangesForTransport = false;
	self.isTransport = true;
	self.attacherVehicleJoint = nil;
	self.mowerIsAttached = false;	
    self.isTurnedOn = false;
    self.wasToFast = false;
	self.isReadyToTransport = true;
	self.soundOffset = 0;
end;

function ExtendedWindrower:delete()
    if self.windrowerSound ~= nil then
        delete(self.windrowerSound);
    end;
	if self.windrowerStartSound ~= nil then
        delete(self.windrowerStartSound);
    end;
	if self.windrowerStopSound ~= nil then
        delete(self.windrowerStopSound);
    end;
end;

function ExtendedWindrower:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ExtendedWindrower:keyEvent(unicode, sym, modifier, isDown)
end;

function ExtendedWindrower:update(dt)

	if self:getIsActive() then
		
		self.wasToFast = false;
		
		for k,v in pairs(self.particleSystems) do
            if self.time > v.disableTime then
                Utils.setEmittingState(v.ps, false);
            end;
        end;
		if self.doChangesForTransport then
			self.timeToNextTine = self.timeToNextTine - dt;
			
			if self.timeToNextTine < 0 then	
				
				self.timeToNextTine = self.timePerTine;
				if self.isTransport then
					local isCheckComplete = true;
					local tineCount = table.getn(self.tines);
					local positionCount = table.getn(self.tinePositions);
					for k, tine in pairs(self.tines) do						
						-- calculate distance to max height
						local x,y,z = getWorldTranslation(tine.node);
						local marker = self.markers[tine.marker];
						width,_,_ = worldToLocal(getParent(marker.node), x,y,z);
						local isVisible = getVisibility(tine.node);						
						if (marker.width * marker.scale) > (width * marker.scale)  and isVisible then
							if self.invisibleTineCount <= positionCount then
								if (k > tineCount/2) or (k <= tineCount/2 and self.invisibleTineCount < positionCount/2) then
									setVisibility(tine.node, false);
									self.invisibleTineCount = self.invisibleTineCount + 1;
									setVisibility(self.tinePositions[self.invisibleTineCount], true);
									isCheckComplete = false;
									break;
								end;
							end;
						end;						
					end;
					
					if isCheckComplete then
						self.doChangesForTransport = false;
					end;
				else
					local isCheckComplete = true;
					for k, tine in pairs(self.tines) do							
						local isVisible = getVisibility(tine.node);		
						if not isVisible then								
							setVisibility(tine.node, true);	
							setVisibility(self.tinePositions[self.invisibleTineCount], false);
							self.invisibleTineCount = self.invisibleTineCount - 1;
							isCheckComplete = false;
							break;
						end;						
					end;	
					if isCheckComplete then
						self.doChangesForTransport = false;
					end;
				end;
			end;
		end;	

		if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
			if not self.isTransport then
				self:setActivity(not self.isTurnedOn);
			end;			
		end;
		if self.isReadyToTransport then
			if InputBinding.hasEvent(InputBinding.EXTENDEDWINDROWER_TRANSPORT) then
				self.isTransport = not self.isTransport;
				if not self.isTransport then
					self.doChangesForTransport = true;
				else
					-- wait for rower animation stop before checking tines
					self.doTransportCheck = true;				
				end;
				if self.isTransport then
					if self.isTurnedOn then
						self:setActivity(false);
					end;
				end;
			end;
		end;
		
		if self.doTransportCheck then
			if not isAnimTrackEnabled(self.animation.animCharSet, 0) then
				self.doChangesForTransport = true;
				self.doTransportCheck = false;
			end;			
		end;		

		if self.isTurnedOn then			
			local toFast = false;						
			toFast = self:doCheckSpeedLimit() and self.lastSpeed*3600 > 31;				
			if not toFast then
				local numDropAreas = table.getn(self.windrowerDropAreas);
				local numAreas = table.getn(self.cuttingAreas);
				local sum = 0;
				local fruitType = FruitUtil.FRUITTYPE_GRASS;					
				local fruitTypeFix = false;
				for i=1, 3 do 					
					if i==2 then
						fruitType = FruitUtil.FRUITTYPE_WHEAT;
					elseif i==3 then
						fruitType = FruitUtil.FRUITTYPE_BARLEY;
					end;						
					for j=1, numAreas do
						local cuttingArea = self.cuttingAreas[j];
						if cuttingArea.isExpanded then
							local x,y,z = getWorldTranslation(cuttingArea.groundReferenceNode);
							local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, 0, z);	
							
							if terrainHeight+self.groundReferenceThreshold >= y then	
								local x,y,z = getWorldTranslation(cuttingArea.start);
								local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
								local x2,y2,z2 = getWorldTranslation(cuttingArea.height);

								local ratio = g_currentMission.windrowCutLongRatio;

								if not fruitTypeFix and i==1 then
									fruitType = FruitUtil.FRUITTYPE_GRASS;
								end;

								local area = Utils.updateFruitCutLongArea(fruitType, x, z, x1, z1, x2, z2, 0)/ratio;
								area = area + Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, 0);
								
								if area == 0 and not fruitTypeFix and i==1 then
									fruitType = FruitUtil.FRUITTYPE_DRYGRASS;
									area = Utils.updateFruitCutLongArea(fruitType, x, z, x1, z1, x2, z2, 0)/ratio;
									area = area + Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, 0);
								end;

								if area > 0 then
									fruitTypeFix = true;
								end;

								if numDropAreas >= numAreas then
									if area > 0 then
										local dropArea = self.windrowerDropAreas[j];
										local x,y,z = getWorldTranslation(dropArea.start);
										local x1,y1,z1 = getWorldTranslation(dropArea.width);
										local x2,y2,z2 = getWorldTranslation(dropArea.height);
										local old, total = Utils.getFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2);
										area = area + old;
										local value = area / total;
										if value < 1 and value > 0.08 then
											value = 1;
										else
											value = math.floor(value + 0.6); -- round, biased to the bigger value
										end;
										if value >= 1 then
											value = math.min(value, g_currentMission.maxWindrowValue);
											Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, value, true);
										end;									
													
										if g_currentMission.environment.lastRainScale <= 0.1 and g_currentMission.environment.timeSinceLastRain > 30 then
											ps = self.particleSystems[1];
											if ps ~= nil then
												ps.disableTime = self.time + 20;
												Utils.setEmittingState(ps.ps, true);
											end;	
										end;
										
									end;
								else
									sum = sum + area;
								end;
							end;
						end;
					end;
					if sum > 0 and numDropAreas > 0 then
						local dropArea = self.windrowerDropAreas[1];
						local x,y,z = getWorldTranslation(dropArea.start);
						local x1,y1,z1 = getWorldTranslation(dropArea.width);
						local x2,y2,z2 = getWorldTranslation(dropArea.height);
						local old, total = Utils.getFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2);
						sum = sum + old;
						local value = math.floor(sum / total + 0.7); -- round, biased to the bigger value
						if value >= 1 then
							value = math.min(value, g_currentMission.maxWindrowValue);
							Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, value, true);
						end;							
					end;
				end;
			end;
            if not self.animationEnabled then
                enableAnimTrack(self.animation.animCharSet, 0);
                self.animationEnabled = true;
			else
				if self.animationSpeed < (self.animation.speedScale - 0.01) then
					local newSpeed = Utils.getMovedLimitedValues({self.animationSpeed}, {self.animation.speedScale}, {0}, 1, self.animation.offsetTime, dt, false);
					self.animationSpeed = newSpeed[1];
					setAnimTrackSpeedScale(self.animation.animCharSet, 0, self.animationSpeed);
				end;
            end;
            if not self.windrowerSoundEnabled and self:getIsActiveForSound() and self.time > self.soundOffset then
                playSample(self.windrowerSound, 0, self.windrowerSoundVolume, 0);
                setSamplePitch(self.windrowerSound, self.windrowerSoundPitchOffset);
                self.windrowerSoundEnabled = true;
            end;

			local areRowerDown = false;
			for _, area in pairs(self.cuttingAreas) do
				areRowerDown = areRowerDown or area.isExpanded;
			end;
            self.wasToFast = toFast and areRowerDown;				
        else
            if self.animationEnabled and self.animationSpeed < 0.01 then
                disableAnimTrack(self.animation.animCharSet, 0);
                self.animationEnabled = false;
			else
				local newSpeed = Utils.getMovedLimitedValues({self.animationSpeed}, {self.animation.speedScale}, {0}, 1, self.animation.offsetTime, dt, true);
				self.animationSpeed = newSpeed[1];
				setAnimTrackSpeedScale(self.animation.animCharSet, 0, self.animationSpeed);
            end;
        end;
	end;
	
	if not self.isTurnedOn and self.windrowerSoundEnabled then
        stopSample(self.windrowerSound);
        self.windrowerSoundEnabled = false;
    end;
	
end;

function ExtendedWindrower:draw()
    if self.isTransport then
        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("KUHNGA_TRANSPORT_OFF"), self.typeDesc), InputBinding.EXTENDEDWINDROWER_TRANSPORT);
    else
		if self.isReadyToTransport then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("KUHNGA_TRANSPORT_ON"), self.typeDesc), InputBinding.EXTENDEDWINDROWER_TRANSPORT);
		end;
		if self.isTurnedOn then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_off_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
	
		else
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_on_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
		end;
		g_currentMission:addExtraPrintText(string.format(g_i18n:getText("EXTENEDROWER_CONTROL"), self.typeDesc) .. g_i18n:getText("Key") .. " " .. InputBinding.getButtonKeyName(InputBinding.EXTENEDROWER_LOWER_ALL) .. "/" .. InputBinding.getButtonKeyName(InputBinding.EXTENEDROWER_LOWER_LEFT) .. "/" .. InputBinding.getButtonKeyName(InputBinding.EXTENEDROWER_LOWER_RIGHT));
    end;

    if self.wasToFast then
        g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "2", InputBinding.getButtonKeyName(InputBinding.SPEED_LEVEL2)), 0.07+0.022, 0.019+0.029);
    end;
end;

function ExtendedWindrower:onDetach()
	if self.deactivateOnDetach then
        ExtendedWindrower.onDeactivate(self);
    end;
end;

function ExtendedWindrower:onDeactivate()
	if self.animationEnabled then
        disableAnimTrack(self.animation.animCharSet, 0);
        self.animationEnabled = false;
    end;
	for k,v in pairs(self.particleSystems) do
        Utils.setEmittingState(v.ps, false);
    end;
    self.isTurnedOn = false;
end;


function ExtendedWindrower:onLeave()
    if self.deactivateOnLeave then
        ExtendedWindrower.onDeactivate(self);
    end;
end;

function ExtendedWindrower:setActivity(turnOn)

	self.isTurnedOn = turnOn;
	
	if self:getIsActiveForSound() then
		local soundOffset = 0;
		if self.isTurnedOn then	
			if self.time < self.soundOffset then
				stopSample(self.windrowerStopSound);
			end;
			if self.windrowerStartSound ~= nil and self:getIsActiveForSound() then
				playSample(self.windrowerStartSound, 1, self.windrowerStartSoundVolume, 0);
				setSamplePitch(self.windrowerStartSound, self.windrowerStartSoundPitchOffset);
				soundOffset = getSampleDuration(self.windrowerStartSound);
			end;					
		else
			if self.time < self.soundOffset then
				stopSample(self.windrowerStartSound);
			else
				if self.windrowerStopSound ~= nil then
					setSamplePitch(self.windrowerStopSound, self.windrowerStopSoundPitchOffset);
					playSample(self.windrowerStopSound, 1, self.windrowerStopSoundVolume, 0);
					soundOffset = getSampleDuration(self.windrowerStopSound);
				end;
			end;
		end;
		self.soundOffset = self.time + soundOffset;
	end;
end;


