--
-- KuhnGA8121
--
-- @author  Manuel Leithner
-- @date  10/08/10
--

KuhnGA8121 = {};

function KuhnGA8121.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(ExtendedWindrower, specializations);
end;

function KuhnGA8121:load(xmlFile)
	self.doJointOrientation = false;
	self.attacherJointCopy = nil;
	self.updateJoint = false;
	
	
	self.steeringAnimations = {};
	self.steeringAnimations.axes = {};
	local i = 0;
    while true do
        local partName = string.format("vehicle.steeringAnimation.axis(%d)", i);
		local axis = {};
        local index = getXMLString(xmlFile, partName .. "#index");
        if index == nil then
            break;
        end;
		axis.node = Utils.indexToObject(self.components, index);
		axis.wheel = self.wheels[getXMLInt(xmlFile, partName .. "#wheelIndex")];
		if axis.wheel ~= nil then
			table.insert(self.steeringAnimations.axes, axis);
		else
			print("Error: Could not found wheel index!");
		end;		
        i = i+1;
	end;
	self.steeringAnimations.middleController = {};
	self.steeringAnimations.middleController.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.steeringAnimation#middleController"));
	self.steeringAnimations.middleController.minTrans = getXMLFloat(xmlFile, "vehicle.steeringAnimation#minTrans");
	self.steeringAnimations.middleController.maxTrans = getXMLFloat(xmlFile, "vehicle.steeringAnimation#maxTrans");
	
	
	self.rowerFoldingParts = {};
	local i = 0;
    while true do
        local partName = string.format("vehicle.rowerFoldingParts.part(%d)", i);
		local part = {};
        local joint = getXMLInt(xmlFile, partName .. ".mainPart#joint");
        if joint == nil then
            break;
        end;
		local mainPart = {};
		mainPart.joint = self.componentJoints[joint];
		mainPart.minRot = {};
		local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, partName .. ".mainPart#minRot"));
        mainPart.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        mainPart.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        mainPart.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
		
		mainPart.maxRot = {};
		x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, partName .. ".mainPart#maxRot"));
        mainPart.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        mainPart.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        mainPart.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
		mainPart.depHydr = {Utils.getVectorFromString(getXMLString(xmlFile, partName .. ".mainPart#dependingHydraulic"))};		
		mainPart.maxLimit = {};
		x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, partName .. ".mainPart#maxLimit"));
        mainPart.maxLimit[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        mainPart.maxLimit[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        mainPart.maxLimit[3] = Utils.degToRad(Utils.getNoNil(z, 0));
		mainPart.currentLimit = {0, 0, 0};
		mainPart.isDown = false;
		mainPart.transNode = Utils.indexToObject(self.components, getXMLString(xmlFile, partName .. ".mainPart#transNode"));
		mainPart.minTrans = {getXMLFloat(xmlFile, partName .. ".mainPart#minTrans")};
		mainPart.maxTrans = {getXMLFloat(xmlFile, partName .. ".mainPart#maxTrans")};
		mainPart.widthRotNode = Utils.indexToObject(self.components, getXMLString(xmlFile, partName .. ".mainPart#widthRotNode"));
		mainPart.widthMinRot = {Utils.degToRad(Utils.getNoNil(getXMLFloat(xmlFile, partName .. ".mainPart#widthMinRot"), 0))};
		mainPart.widthMaxRot = {Utils.degToRad(Utils.getNoNil(getXMLFloat(xmlFile, partName .. ".mainPart#widthMaxRot"), 0))};
		local bar = {};
		bar.node = Utils.indexToObject(self.components, getXMLString(xmlFile, partName .. ".bar#node"));
		bar.minRot = {};
		x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, partName .. ".bar#minRot"));
        bar.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        bar.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        bar.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
		bar.maxRot = {};
		x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, partName .. ".bar#maxRot"));
        bar.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        bar.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        bar.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
		
		local rower = {};
		rower.joint = self.componentJoints[getXMLInt(xmlFile, partName .. ".rower#joint")];
		rower.maxLimit = {};
		x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, partName .. ".rower#maxLimit"));
        rower.maxLimit[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        rower.maxLimit[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        rower.maxLimit[3] = Utils.degToRad(Utils.getNoNil(z, 0));
		rower.cuttingArea = getXMLInt(xmlFile, partName .. ".rower#cuttingArea");
		rower.currentLimit = {0, 0, 0};
		
		part.rower = rower;
		part.bar = bar;
		part.mainPart = mainPart;
		part.moveTime = Utils.getNoNil(getXMLFloat(xmlFile, partName .. "#moveTime"),5)*1000;
		part.workingTime = Utils.getNoNil(getXMLFloat(xmlFile, partName .. "#workingTime"),2)*1000;
		part.isDown = false;
		table.insert(self.rowerFoldingParts, part);		
        i = i+1;
	end;	
	
	self.updateSteeringAxleAngle = false;
end;

function KuhnGA8121:delete()
end;

function KuhnGA8121:mouseEvent(posX, posY, isDown, isUp, button)
end;

function KuhnGA8121:keyEvent(unicode, sym, modifier, isDown)

end;

function KuhnGA8121:update(dt)

	if self.doJointOrientation and self.attacherVehicle ~= nil then
		for k,v in pairs(self.attacherVehicle.attachedImplements) do
			if v.object == self then
				local joint = self.attacherVehicle.attacherJoints[v.jointDescIndex];
				self.attacherJointCopy = {};
				self.attacherJointCopy.joint = joint;	
				self.attacherJointCopy.minRotCopy = joint.minRot;
				self.attacherJointCopy.maxRotCopy = joint.maxRot;
				joint.minRot = joint.maxRot;
			end;
		end;
		self.doJointOrientation = false;
	end;
		
	if self:getIsActive() then
		self.isModifierActive = false;
		local hydraulicMovingDirection = 0;
		
		if self.attacherVehicle ~= nil then
			if Input.isKeyPressed(Input.KEY_shift) then	
				if InputBinding.isPressed(InputBinding.KUHN_LOWER_HYDRAULIC) then
					hydraulicMovingDirection = -1;
				elseif InputBinding.isPressed(InputBinding.KUHN_LIFT_HYDRAULIC) then
					hydraulicMovingDirection = 1;
				end;
			end;
			self.isReadyToTransport = true;
			for _, part in pairs(self.rowerFoldingParts) do
				self.isReadyToTransport = self.isReadyToTransport and not part.mainPart.isDown;
			end;
			if self.isReadyToTransport then
				if InputBinding.hasEvent(InputBinding.EXTENDEDWINDROWER_TRANSPORT) then
					for _, part in pairs(self.rowerFoldingParts) do
						part.isDown = not part.isDown;
					end;	
				end;
			end;
			if not self.isTransport then
				if InputBinding.hasEvent(InputBinding.EXTENEDROWER_LOWER_LEFT) then
					self.rowerFoldingParts[2].mainPart.isDown = not self.rowerFoldingParts[2].mainPart.isDown
				end;
				if InputBinding.hasEvent(InputBinding.EXTENEDROWER_LOWER_RIGHT) then
					self.rowerFoldingParts[1].mainPart.isDown = not self.rowerFoldingParts[1].mainPart.isDown;
				end;
				if InputBinding.hasEvent(InputBinding.EXTENEDROWER_LOWER_ALL) then
					local isDown = self.rowerFoldingParts[1].mainPart.isDown or self.rowerFoldingParts[2].mainPart.isDown;
					for _, part in pairs(self.rowerFoldingParts) do
						part.mainPart.isDown = not isDown;
					end;	
				end;
			end;
		end;		
		
		if self.attacherVehicle ~= nil then	
			local x,y,z = worldDirectionToLocal(self.steeringAxleNode, localDirectionToWorld(self.attacherVehicle.steeringAxleNode, 0, 0, 1));
			local dot = z; -- 0*x + z*1;
			dot = dot / Utils.vector2Length(x,z);
			local angle = math.acos(dot);
			if x < 0 then
				angle = -angle;
			end;
			local startSpeed = self.steeringAxleAngleScaleStart;
			local endSpeed = self.steeringAxleAngleScaleEnd;
			local scale = Utils.clamp(1 + (self.lastSpeed*3600-startSpeed) * 1.0/(startSpeed-endSpeed), 0, 1);
			self.steeringAxleAngle = angle*scale;

			for _, axis in pairs(self.steeringAnimations.axes) do
				setRotation(axis.node, getRotation(axis.wheel.repr));
			end;
			
			local x,y,z = getRotation(self.wheels[1].repr);
			local maxAngle = self.wheels[1].steeringAxleRotMax;
			local minAngle = self.wheels[1].steeringAxleRotMin;
			local percentage = (y - minAngle) / (maxAngle + math.abs(minAngle));
			local a,b,c = getTranslation(self.steeringAnimations.middleController.node);
			c = self.steeringAnimations.middleController.minTrans + percentage * (self.steeringAnimations.middleController.maxTrans - self.steeringAnimations.middleController.minTrans);
			setTranslation(self.steeringAnimations.middleController.node, a, b, c);
		
			local jointDesc = self.attacherJointCopy.joint;
			setJointRotationLimit(jointDesc.jointIndex, 2, true, math.rad(-50), math.rad(50));
			jointDesc.moveDown = false;
			self.movingDirection = 1;
			if hydraulicMovingDirection ~= 0 then				
				if jointDesc.rotationNode ~= nil then
					local x, y, z = getRotation(jointDesc.rotationNode);
					local rot = {x,y,z};
					local newRot = Utils.getMovedLimitedValues(rot, self.attacherJointCopy.maxRotCopy, self.attacherJointCopy.minRotCopy, 3, jointDesc.moveTime, dt, hydraulicMovingDirection == 1);
					setRotation(jointDesc.rotationNode, unpack(newRot));
					jointDesc.maxRot = newRot;
					jointDesc.minRot = newRot;
					setJointFrame(jointDesc.jointIndex, 0, jointDesc.jointTransform);
				end;
			end;
			
			for _, foldingPart in pairs(self.rowerFoldingParts) do
				local mainPart = foldingPart.mainPart;
				local x, y, z = getRotation(mainPart.joint.jointNode);
				local rot = {x,y,z};
				local newRot = Utils.getMovedLimitedValues(rot, mainPart.maxRot, mainPart.minRot, 3, foldingPart.moveTime, dt, not foldingPart.isDown);
				setRotation(mainPart.joint.jointNode, unpack(newRot));
				setJointFrame(mainPart.joint.jointIndex, 0, mainPart.joint.jointNode);
				local bar = foldingPart.bar;
				x, y, z = getRotation(bar.node);
				rot = {x,y,z};
				newRot = Utils.getMovedLimitedValues(rot, bar.maxRot, bar.minRot, 3, foldingPart.moveTime, dt, not foldingPart.isDown);
				setRotation(bar.node, unpack(newRot));
				
				x, y, z = getTranslation(mainPart.transNode);
				x = Utils.getMovedLimitedValues({x}, mainPart.maxTrans, mainPart.minTrans, 1, foldingPart.moveTime, dt, not foldingPart.isDown);
				setTranslation(mainPart.transNode, x[1],y,z);
				x, y, z = getRotation(mainPart.widthRotNode);
				x = Utils.getMovedLimitedValues({x}, mainPart.widthMaxRot, mainPart.widthMinRot, 1, foldingPart.moveTime, dt, not foldingPart.isDown);
				setRotation(mainPart.widthRotNode, x[1],y,z);
				
				local rower = foldingPart.rower;
				setJointFrame(rower.joint.jointIndex, 0, rower.joint.jointNode);

				local newArmLimit = Utils.getMovedLimitedValues(mainPart.currentLimit, mainPart.maxLimit, {0,0,0}, 3, foldingPart.workingTime, dt, not mainPart.isDown);
				local newRowerLimit = Utils.getMovedLimitedValues(rower.currentLimit, rower.maxLimit, {0,0,0}, 3, foldingPart.workingTime, dt, not mainPart.isDown);
				for i=1, 3 do
					setJointRotationLimit(mainPart.joint.jointIndex, i-1, true, -newArmLimit[i], newArmLimit[i]);
					setJointRotationLimit(rower.joint.jointIndex, i-1, true, -newRowerLimit[i], newRowerLimit[i]);
				end;
				mainPart.currentLimit = newArmLimit;
				rower.currentLimit = newRowerLimit;

				local isExpanded = true;
                for i=1, 3 do
					isExpanded = isExpanded and newRowerLimit[i] >= rower.maxLimit[i] * 0.5;
                end;	
				self.cuttingAreas[rower.cuttingArea].isExpanded = isExpanded;
				
				-- check!!!
				for _, index in pairs(mainPart.depHydr) do
					local hydraulic = self.hydraulics[index];
					if hydraulic ~= nil then
						hydraulic.needsUpdate = true;
					end;
				end;
			end;
		else
			self.steeringAxleAngle = 0;
		end;		
	end;
end;


function KuhnGA8121:draw()	
    if self.isTransport then
        g_currentMission:addExtraPrintText(string.format(g_i18n:getText("EXTENEDROWER_LIFT_HYDRAULIC"), self.typeDesc) .. g_i18n:getText("Key") .. " " .. g_i18n:getText("EXTENEDROWER_MODIFIER") .. "+" .. InputBinding.getButtonKeyName(InputBinding.KUHN_LIFT_HYDRAULIC) .. "/" .. InputBinding.getButtonKeyName(InputBinding.KUHN_LOWER_HYDRAULIC));
    end;
end;

function KuhnGA8121:onAttach(attacherVehicle)
	self.doJointOrientation = true;	
	setJointRotationLimit(self.componentJoints[1].jointIndex, 1, true, math.rad(-80), math.rad(80));
	setJointRotationLimit(self.componentJoints[1].jointIndex, 2, true, math.rad(-5), math.rad(5));
end;

function KuhnGA8121:onDetach()
	if self.attacherJointCopy ~= nil then
		local jointDesc = self.attacherJointCopy.joint;
		jointDesc.maxRot = self.attacherJointCopy.maxRotCopy;
		jointDesc.minRot = self.attacherJointCopy.minRotCopy;
	end;
	setJointRotationLimit(self.componentJoints[1].jointIndex, 1, true, math.rad(0), math.rad(0));
	setJointRotationLimit(self.componentJoints[1].jointIndex, 2, true, math.rad(0), math.rad(0));
end;