--
-- Supportbar
-- Specialization for Supportbar functionality
--
-- @author  Manuel Leithner
-- @date  10/08/10
-- @version v2.0
--

Supportbar = {};

function Supportbar.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Attachable, specializations);
end;

function Supportbar:load(xmlFile)
	
	self.supportbar = {};
	self.supportbar.jointIndex = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.supportbar#jointIndex"), 0);
	
	if self.supportbar.jointIndex < 0 or self.supportbar.jointIndex > table.getn(self.componentJoints) then
		self.supportbar.jointIndex = 0;
		print("Supportbar-Error: Illegal joint index!");
	end;
		
	self.supportbar.rotationNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.supportbar#rotationNode"));	
	
	local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile,  "vehicle.supportbar#minRot"));
	self.supportbar.minRot = {};
	self.supportbar.minRot[1] = math.rad(Utils.getNoNil(x, 0));
	self.supportbar.minRot[2] = math.rad(Utils.getNoNil(y, 0));
	self.supportbar.minRot[3] = math.rad(Utils.getNoNil(z, 0));	
	x, y, z = Utils.getVectorFromString(getXMLString(xmlFile,  "vehicle.supportbar#maxRot"));
	self.supportbar.maxRot = {};
	self.supportbar.maxRot[1] = math.rad(Utils.getNoNil(x, 0));
	self.supportbar.maxRot[2] = math.rad(Utils.getNoNil(y, 0));
	self.supportbar.maxRot[3] = math.rad(Utils.getNoNil(z, 0));		
	self.supportbar.rotTime = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.supportbar#rotTime"), 1) * 1000;

	
	self.supportbar.jointTransNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.supportbar#jointTransNode"));	
	if self.supportbar.jointTransNode ~= nil then
		local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile,  "vehicle.supportbar#jointTransMin"));
		self.supportbar.minJointTrans = {};
		self.supportbar.minJointTrans[1] = Utils.getNoNil(x, 0);
		self.supportbar.minJointTrans[2] = Utils.getNoNil(y, 0);
		self.supportbar.minJointTrans[3] = Utils.getNoNil(z, 0);
		x, y, z = Utils.getVectorFromString(getXMLString(xmlFile,  "vehicle.supportbar#jointTransMax"));
		self.supportbar.maxJointTrans = {};
		self.supportbar.maxJointTrans[1] = Utils.getNoNil(x, 0);
		self.supportbar.maxJointTrans[2] = Utils.getNoNil(y, 0);
		self.supportbar.maxJointTrans[3] = Utils.getNoNil(z, 0);	
		self.supportbar.jointTransTime = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.supportbar#jointTransTime"), 1) * 1000;	
	end;
	
	
	self.supportbar.isExpanded = true;
	self.supportbar.isActionFinished = true;
	self.renewCollision = false;
	self.runOnce = true;
end;

function Supportbar:delete()
end;

function Supportbar:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Supportbar:keyEvent(unicode, sym, modifier, isDown)
end;

function Supportbar:update(dt)		
	
	if not self.supportbar.isActionFinished then	
		if self.supportbar.rotationNode ~= nil then
			self.renewCollision = false;
			local x, y, z = getRotation(self.supportbar.rotationNode);
			local rot = {x,y,z};
			local newRot = Utils.getMovedLimitedValues(rot, self.supportbar.maxRot, self.supportbar.minRot, 3, self.supportbar.rotTime, dt, self.supportbar.isExpanded);
			setRotation(self.supportbar.rotationNode, unpack(newRot));			
			for i=1, 3 do
				if math.abs(newRot[i] - rot[i]) > 0.001 then
					self.renewCollision = true;
					break;
				end;
			end;

			local x, y, z = getTranslation(self.supportbar.jointTransNode);
			local newTrans = Utils.getMovedLimitedValues({x,y,z}, self.supportbar.maxJointTrans, self.supportbar.minJointTrans, 3, self.supportbar.jointTransTime, dt, self.supportbar.isExpanded);
			setTranslation(self.supportbar.jointTransNode, unpack(newTrans));

			self.supportbar.isActionFinished = true;
			local x,y,z = getRotation(self.supportbar.rotationNode);
			local rot = {x,y,z};
			local finishRot = self.supportbar.minRot;
			if not self.supportbar.isExpanded then
				finishRot = self.supportbar.maxRot;		
			end;
			if self.supportbar.jointIndex > 0 and (self.renewCollision or self.runOnce) then
				local index = self.supportbar.jointIndex;			
				setJointFrame(self.componentJoints[index].jointIndex, 0, self.supportbar.jointTransNode);
				self.runOnce = false;
			end;
			
			for i=1, 3 do
				if math.abs(finishRot[i] - rot[i]) > 0.001 then
					self.supportbar.isActionFinished = false;
					break;
				end;
			end;	
		end;
	end;
end;

function Supportbar:draw()
end;

function Supportbar:onAttach(attacherVehicle)
	self.supportbar.isExpanded = true;
	self.supportbar.isActionFinished = false;
end;

function Supportbar:onDetach()
	self.supportbar.isExpanded = false;
	self.supportbar.isActionFinished = false;
end;





